#ifndef CGDATAPROVIDER_H
#define CGDATAPROVIDER_H

#include <CoreGraphics/CGBase.h>
#include <CoreFoundation/CFURL.h>
#include <stddef.h>

/*CGDataProviderRef

Defines an opaque type that supplies Quartz with data.

typedef struct CGDataProvider *CGDataProviderRef;
Discussion

Some Quartz routines supply blocks of data to your program. Rather than reading through a raw memory buffer, data provider objects of type CGDataProviderRef allow you to supply Quartz functions with data. 

In Mac OS X version 10.2 and later, CGDataProviderRef is derived from CFTypeRef and inherits the properties that all Core Foundation types have in common. For more information, see CFType Reference.
Availability
Available in Mac OS X v10.0 and later.*/
typedef struct CGDataProvider *CGDataProviderRef;

/*CGDataProviderGetBytePointerCallback

A callback function that returns a generic pointer to the provider data.

void * (*CGDataProviderGetBytePointerCallback) (
   void *info
);
   


If you name your function MyProviderGetBytePointer, you would declare it like this:

void *MyProviderGetBytePointer (
   void *info
);
Parameters
info

A generic pointer to private data shared among your callback functions. This is the same pointer you supplied to CGDataProviderCreateDirectAccess.
Return Value

A generic pointer to your provider data. By suppling this pointer, you are giving Quartz read-only access to both the pointer and the underlying provider data. You must not move or modify the provider data until Quartz calls your CGDataProviderReleaseBytePointerCallback function.
Discussion

When Quartz needs direct access to your provider data, this function is called.

For information on how to associate your function with a direct-access data provider, see CGDataProviderCreateDirectAccess and CGDataProviderDirectAccessCallbacks.*/
typedef void * (*CGDataProviderGetBytePointerCallback) (
   void *info
);

/*CGDataProviderReleaseBytePointerCallback

A callback function that releases the pointer Quartz obtained by calling CGDataProviderGetBytePointerCallback.

typedef void (*CGDataProviderReleaseBytePointerCallback) (
   void *info,
   const void *pointer
);
   


If you name your function MyProviderReleaseBytePointer, you would declare it like this:

void MyProviderReleaseBytePointer (
   void *info,
   const void *pointer
);
Parameters
info

A generic pointer to private data shared among your callback functions. This is the same pointer you supplied to CGDataProviderCreateDirectAccess.
pointer

A pointer to your provider data. This is the same pointer you returned in CGDataProviderGetBytePointerCallback.
Discussion

When Quartz no longer needs direct access to your provider data, your function is called. You may safely modify, move, or release your provider data at this time.

For information on how to associate your function with a direct-access data provider, see CGDataProviderCreateDirectAccess and CGDataProviderDirectAccessCallbacks.*/
typedef void (*CGDataProviderReleaseBytePointerCallback) (
   void *info,
   const void *pointer
);

/*CGDataProviderGetBytesAtOffsetCallback

A callback function that copies data from the provider into a Quartz buffer.

typedef size_t (*CGDataProviderGetBytesAtOffsetCallback) (
   void *info,
   void *buffer,
   size_t offset,
   size_t count
);
   


If you name your function MyProviderGetBytesWithOffset, you would declare it like this:

size_t MyProviderGetBytesWithOffset (
   void *info,
   void *buffer,
   size_t offset,
   size_t count
);
Parameters
info

A generic pointer to private data shared among your callback functions. This is the same pointer you supplied to CGDataProviderCreateDirectAccess.
buffer

The Quartz-supplied buffer into which you copy the specified number of bytes.
offset

Specifies the relative location in the data provider at which to begin copying data.
count

The number of bytes to copy.
Return Value

The number of bytes copied. If no more data can be written to the buffer, you should return 0.
Discussion

When Quartz is ready to receive data from the provider, your function is called.

For information on how to associate your function with a direct-access data provider, see CGDataProviderCreateDirectAccess and CGDataProviderDirectAccessCallbacks.*/
typedef size_t (*CGDataProviderGetBytesAtOffsetCallback) (
   void *info,
   void *buffer,
   size_t offset,
   size_t count
);

/*CGDataProviderReleaseInfoCallback

A callback function that releases any private data or resources associated with the data provider.

void (*CGDataProviderReleaseInfoCallback) (
   void *info
);
   


If you name your function MyProviderReleaseInfo, you would declare it like this:

void MyProviderReleaseInfo (
   void *info
);
Parameters
info

A generic pointer to private information shared among your callback functions. This is the same pointer you supplied to CGDataProviderCreate.
Discussion

When Quartz frees a data provider that has an associated release function, the release function is called.

For information on how to associate your callback function with a data provider, see CGDataProviderCreate and CGDataProviderCallbacks.*/

typedef void (*CGDataProviderReleaseInfoCallback) (
   void *info
);
   
/*CGDataProviderDirectAccessCallbacks

Defines pointers to client-defined callback functions that manage the sending of data for a direct-access data provider.

struct CGDataProviderDirectAccessCallbacks {
   CGDataProviderGetBytePointerCallback getBytePointer;
   CGDataProviderReleaseBytePointerCallback releaseBytePointer;
   CGDataProviderGetBytesAtOffsetCallback getBytes;
   CGDataProviderReleaseInfoCallback releaseProvider;
};
typedef struct CGDataProviderDirectAccessCallbacks  CGDataProviderDirectAccessCallbacks;

Fields
getBytePointer

A pointer to a function that returns a pointer to the provider’s data. For more information, see CGDataProviderGetBytePointerCallback.
releaseBytePointer

A pointer to a function that Quartz calls to release a pointer to the provider’s data. For more information, see CGDataProviderReleaseBytePointerCallback.
getBytes

A pointer to a function that copies data from the provider. For more information, see CGDataProviderGetBytesAtOffsetCallback.
releaseProvider

A pointer to a function that handles clean-up for the data provider, or NULL. For more information, see CGDataProviderReleaseInfoCallback.
Discussion

You supply a CGDataProviderDirectAccessCallbacks structure to the function CGDataProviderCreateDirectAccess to create a data provider for direct access. The functions specified by the CGDataProviderDirectAccessCallbacks structure are responsible for copying data a block at a time to a memory buffer for Quartz to use. The functions are also responsible for handling the data provider’s basic memory management. For the callback to work, one of the getBytePointer and getBytes parameters must be non-NULL. If both are non-NULL, then getBytePointer is used to access the data.
Availability
Available in Mac OS X v10.0 and later.*/


struct CGDataProviderDirectAccessCallbacks {
   CGDataProviderGetBytePointerCallback getBytePointer;
   CGDataProviderReleaseBytePointerCallback releaseBytePointer;
   CGDataProviderGetBytesAtOffsetCallback getBytes;
   CGDataProviderReleaseInfoCallback releaseProvider;
};
typedef struct CGDataProviderDirectAccessCallbacks  CGDataProviderDirectAccessCallbacks;

//functions
/*CGDataProviderRelease

Decrements the retain count of a data provider.

void CGDataProviderRelease (
   CGDataProviderRef provider
);
Parameters
provider

The data provider to release. 
Discussion

This function is equivalent to CFRelease, except that it does not cause an error if the provider parameter is NULL.
Availability
Available in Mac OS X version 10.0 and later.
Declared In
CGDataProvider.h*/
void CGDataProviderRelease (
   CGDataProviderRef provider
);

/*CGDataProviderCreateWithData

Creates a Quartz direct-access data provider that uses data your program supplies.

CGDataProviderRef CGDataProviderCreateWithData (
   void *info,
   const void *data,
   size_t size,
   void (*releaseData)(void *info, const void *data, size_t size)
);
Parameters
info

A pointer to data of any type, or NULL. When Quartz calls the function specified in the releaseData parameter, Quartz sends it this pointer as its first argument.
data

A pointer to the array of data that the provider contains.
size

A value that specifies the number of bytes that the data provider contains.
releaseData

A pointer to a release callback for the data provider, or NULL. Your release function is called when Quartz frees the data provider. For more information, see CGDataProviderReleaseDataCallback.
Return Value

A new data provider. You are responsible for releasing this object using CGDataProviderRelease.
Discussion

You use this function to create a direct-access data provider that uses callback functions to read data from your program an entire block at one time. 
Availability
Available in Mac OS X version 10.0 and later.
Declared In
CGDataProvider.h*/
CGDataProviderRef CGDataProviderCreateWithData (
   void *info,
   const void *data,
   size_t size,
   void (*releaseData)(void *info, const void *data, size_t size)
);

/*CGDataProviderCreateWithURL

Creates a Quartz direct-access data provider that uses a URL to supply data.

CGDataProviderRef CGDataProviderCreateWithURL (
   CFURLRef url
);
Parameters
url

A CFURL object to use for the data provider. When you supply Quartz data via the provider, it reads the data from the URL address. 
Return Value

A new data provider. You are responsible for releasing this object using CGDataProviderRelease.
Discussion

You use this function to create a direct-access data provider that supplies data from a URL. When you supply Quartz with a direct-access data provider, Quartz obtains data from your program in a single entire block.
Availability
Available in Mac OS X version 10.0 and later.
Declared In
CGDataProvider.h*/

CGDataProviderRef CGDataProviderCreateWithURL (
   CFURLRef url
);

/*CGDataProviderCreateDirectAccess

Creates a Quartz direct-access data provider.

CGDataProviderRef CGDataProviderCreateDirectAccess (
   void *info,
   size_t size,
   const CGDataProviderDirectAccessCallbacks *callbacks
);
Parameters
info

A pointer to data of any type or NULL. When Quartz calls the functions specified in the callbacks parameter, it sends each of the functions this pointer.
size

A value that specifies the number of bytes that the data provider contains.
callbacks

A pointer to a CGDataProviderDirectAccessCallbacks structure that specifies the callback functions you implement to handle the data provider’s basic memory management. For a complete description, see CGDataProviderDirectAccessCallbacks.
Return Value

A new data provider. You are responsible for releasing this object using CGDataProviderRelease.
Discussion

You use this function to create a direct-access data provider that uses callback functions to read data from your program in a single block. 
Availability
Available in Mac OS X version 10.0 and later.
Declared In
CGDataProvider.h*/
CGDataProviderRef CGDataProviderCreateDirectAccess (
   void *info,
   size_t size,
   const CGDataProviderDirectAccessCallbacks *callbacks
);

#endif
