#ifndef DIRSERVICESUTILS_H
#define	 DIRSERVICESUTILS_H

#include <stdarg.h>

#include <AvailabilityMacros.h>
#include <DirectoryService/DirServicesTypes.h>
#include <CoreFoundation/CoreFoundation.h>

#ifdef __cplusplus
extern "C" {
#endif


/*dsAddAttributeValue

Adds a value to an attribute.

tDirStatus dsAddAttributeValue (
   tRecordReference inRecordReference,
   tDataNodePtr inAttributeType,
   tDataNodePtr inAttributeValue
);
Parameters
inRecordReference

On input, value of type tRecordReference obtained by previously calling dsOpenRecord that represents the record having an attribute to which a value is to be appended.
inAttributeType

On input, a value of type tDataNodePtr that points to a tDataNode structure allocated by calling dsDataNodeAllocateBlock or dsDataNodeAllocateString that contains the type of the attribute to which a value is to be added.
inAttributeValue

On input, a value of type tDataNodePtr that points to a tDataNode structure allocated by calling dsDataNodeAllocateBlock or dsDataNodeAllocateString that contains the value that is to be added.
function result

A value of type tDirStatus indicating success (eDSNoErr) or an error. For a list of possible result codes, see “Result Codes.”
Discussion

This function adds the specified value to the specified attribute. The attribute must be capable of having more than one value.

To change the value of an attribute, call dsSetAttributeValue or dsSetAttributeValues.
Availability
Available in Mac OS X v10.0 and later.
Declared In
DirServices.h*/
tDirStatus dsAddAttributeValue (
   tRecordReference inRecordReference,
   tDataNodePtr inAttributeType,
   tDataNodePtr inAttributeValue
);

/*dsDataBufferAllocate

Allocates an Open Directory data buffer.

tDataBufferPtr dsDataBufferAllocate (
   tDirReference inDirReference,
   UInt32 inBufferSize
);

Parameters
inDirReference

On input, a value of type tDirReference obtained by calling dsOpenDirService or dsOpenDirServiceProxy, or zero.
inBufferSize

On input, a value of type unsigned long that specifies the length of the buffer that is to be allocated.
function result

A value of type tDataBufferPtr that points to the allocated tDataBuffer structure.
Discussion

The utility function allocates an Open Directory data buffer of the specified size and returns a value that points to the allocated buffer.

Open Directory data buffers are used by many Open Directory functions to exchange information between an Open Directory client application and an Open Directory plug-in.

When you no longer need the data buffer, call dsDataBufferDeAllocate to deallocate the memory that is associated with it.
Availability
Available in Mac OS X v10.0 and later.
Declared In
DirServicesUtils.h*/
tDataBufferPtr dsDataBufferAllocate (
   tDirReference inDirReference,
   UInt32 inBufferSize
);


/*dsDataBufferDeAllocate

Deallocates an Open Directory data buffer.

tDirStatus dsDataBufferDeAllocate (
   tDirReference inDirReference,
   tDataBufferPtr inDataBufferPtr
);
Parameters
inDirReference

On input, a value of type tDirReference obtained by calling dsOpenDirService or dsOpenDirServiceProxy for which a data buffer is to be deallocated, or zero.
inDataBufferPtr

A value of type tDataBufferPtr that points to the tDataBuffer structure that is to be deallocated.
function result

A value of type tDirStatus indicating success (eDSNoErr) or an error. For a list of possible result codes, see “Result Codes.”
Discussion

This utility function appends the specified string to the specified data list.
Availability
Available in Mac OS X v10.0 and later.
Declared In
DirServicesUtils.h*/
tDirStatus dsDataBufferDeAllocate (
   tDirReference inDirReference,
   tDataBufferPtr inDataBufferPtr
);

/*dsDataListDeAllocate

tDirStatus dsDataListDeAllocate (
    tDirReference inDirReference, 
    tDataListPtr inDataList, 
    dsBool inDeAllocateNodesFlag );  
Discussion 

Please Note: ******** API Change Soon ******** dsDataListDeAllocate() will be soon be obsoleted and no longer supported. Please discontinue using it and instead use --> dsDataListDeallocate() ( <-lower case 'a' ).*/

tDirStatus dsDataListDeAllocate (
    tDirReference inDirReference, 
    tDataListPtr inDataList, 
    dsBool inDeAllocateNodesFlag );  

/*dsDataListDeallocate

tDirStatus dsDataListDeallocate (
    tDirReference inDirReference, 
    tDataListPtr inDataList );  
Discussion 

Note that if the tDataListPtr is a heap based tDataList rather than stack based, that you must also call free() to release the memory for the head of the list after calling dsDataListDeallocate().*/
tDirStatus dsDataListDeallocate (
    tDirReference inDirReference, 
    tDataListPtr inDataList );  

/*dsAppendAuthBuffer


Appends a list of items to an existing buffer. 

tDirStatus dsAppendAuthBuffer (
    tDataBufferPtr inOutAuthBuffer, 
    UInt32 inCount, 
    UInt32 inLen, 
    const void *inData,
    ... ) ;  
Parameters
inOutAuthBuffer
pass in a preallocated buffer.
inCount
the number of length/data pairs on the stack
inLen
the length of one buffer item
inData
a pointer to inLen bytes of data
Discussion 

Use this function as a convenient way to compose the buffer for calls to dsDoDirNodeAuth(). 

Availability
Introduced in Mac OS X v10.5.*/
tDirStatus dsAppendAuthBuffer (
    tDataBufferPtr inOutAuthBuffer, 
    UInt32 inCount, 
    UInt32 inLen, 
    const void *inData,
    ... ) ;  

/*dsFillAuthBuffer


Fills a buffer with a list of items. 

tDirStatus dsFillAuthBuffer (
    tDataBufferPtr inOutAuthBuffer, 
    UInt32 inCount, 
    UInt32 inLen, 
    const void *inData,
    ... ) ;  
Parameters
inOutAuthBuffer
pass in a preallocated buffer to be filled.
inCount
the number of length/data pairs on the stack
inLen
the length of one buffer item
inData
a pointer to inLen bytes of data
Discussion 

Use this function as a convenient way to compose the buffer for calls to dsDoDirNodeAuth(). 

Availability
Introduced in Mac OS X v10.5.*/
tDirStatus dsFillAuthBuffer (
    tDataBufferPtr inOutAuthBuffer, 
    UInt32 inCount, 
    UInt32 inLen, 
    const void *inData,
    ... ) ;  

/*dsBuildListFromStrings

Builds a data list from strings.

tDataListPtr dsBuildListFromStrings(
tDirReference inDirReference,
char *in1stCString,
...);
Parameters
inDirReference

On input, a value of type tDirReference obtained by calling dsOpenDirService or dsOpenDirServiceProxy representing the Open Directory session for which the data list is being built, or zero.
in1stCString

On input, a pointer to a null-terminated string containing data in UTF-8 format that is to be added to the data list. The in1stCString parameter may be followed by one or more parameters of type char *, each pointing to a C string containing data in UTF-8 format that is to be added to the data list.
function result

A value of type tDataListPtr that points to the tDataList structure that has been created.
Discussion

This utility function uses one or more null-terminated strings to build a data list and returns a pointer to it.

When you no longer need the data list, call dsDataListDeallocate t o release the memory associated with it. If the data list is a heap-based data list, you also need to call free().
Availability
Available in Mac OS X v10.0 and later.
Declared In
DirectoryService/DirServices.h*/
tDataListPtr dsBuildListFromStrings(
tDirReference inDirReference,
const char *in1stCString,
...);

/*dsParseAuthAuthority


tDirStatus dsParseAuthAuthority (
    const char *inAuthAuthority,
    char **outVersion,
    char **outAuthTag,
    char **outAuthData ) ;  
Discussion 

Pass in a complete authentication authority attribute format is version;tag;data retrieve version, tag, and data as separate strings Memory for the char** parameters is the responsibility of the client and can be cleaned up using free().

Availability
Introduced in Mac OS X v10.4.*/
tDirStatus dsParseAuthAuthority (
    const char *inAuthAuthority,
    char **outVersion,
    char **outAuthTag,
    char **outAuthData ) ;  

/*dsAllocAttributeValueEntry


tAttributeValueEntryPtr dsAllocAttributeValueEntry (
    tDirReference inDirRef, 
    UInt32 inAttrValueID, 
    void *inAttrValueData, 
    UInt32 inAttrValueDataLen );  */

tAttributeValueEntryPtr dsAllocAttributeValueEntry (
    tDirReference inDirRef, 
    UInt32 inAttrValueID, 
    void *inAttrValueData, 
    UInt32 inAttrValueDataLen ); 

/*dsGetRecordNameFromEntry

Gets the name of a record from a record entry structure.

tDirStatus dsGetRecordNameFromEntry (
   tRecordEntryPtr inRecEntryPtr,
   char **outRecName
);

Parameters
inRecEntryPtr

On input, a value of type tRecordEntryPtr that points to the tRecordEntry structure that contains the name that is to be obtained.
outRecName

On output, a pointer to a value that points to a character string containing the record’s name in UTF-8 format.
function result

A value of type tDirStatus indicating success (eDSNoErr) or an error. For a list of possible result codes, see “Result Codes.”
Discussion

This utility function gets the name of the record in a record entry structure.
Availability
Available in Mac OS X v10.0 and later.
Declared In
DirServicesUtils.h*/
tDirStatus dsGetRecordNameFromEntry (
   tRecordEntryPtr inRecEntryPtr,
   char **outRecName
);

/*dsGetRecordTypeFromEntry

Gets the type of a record from a record entry structure.

tDirStatus dsGetRecordTypeFromEntry (
   tRecordEntryPtr inRecEntryPtr,
   char **outRecType
);

Parameters
inRecEntryPtr

On input, a value of type tRecordEntryPtr that points to the tRecordEntry structure that contains the type that is to be obtained.
outRecType

On output, a pointer to a value that points to a character string containing the record’s type.
function result

A value of type tDirStatus indicating success (eDSNoErr) or an error. For a list of possible result codes, see “Result Codes.”
Discussion

This utility function gets the type of the record in a record entry structure.
Availability
Available in Mac OS X v10.0 and later.
Declared In
DirServicesUtils.h*/
tDirStatus dsGetRecordTypeFromEntry (
   tRecordEntryPtr inRecEntryPtr,
   char **outRecType
);

/*dsDataListAllocate

Allocates a data list.

tDataListPtr dsDataListAllocate (
   tDirReference inDirReference
);

Parameters
inDirReference

On input, a value of type tDirReference obtained by calling dsOpenDirService or dsOpenDirServiceProxy representing the Open Directory session for which the data list is to be allocated, or zero.
function result

A value of type tDataListPtr that points to the allocated tDataList structure. If this function cannot allocate the data list, it returns NULL.
Discussion

This utility function allocates an empty data list and returns a value of type tDataListPtr that points to it.

Many Open Directory functions return information in a data list and receive information in a data list, such as dsFindDirNodes, dsGetDirNodeInfo, dsGetRecordList, and dsDoAttributeValueSearch.

To add data to the data list, call dsBuildListFromNodesAlloc or dsBuildListFromStringsAlloc.

When you no longer need the data list, call dsDataListDeallocate to release the memory associated with it.
Availability
Available in Mac OS X v10.0 and later.
Declared In
DirServicesUtils.h*/
tDataListPtr dsDataListAllocate (
   tDirReference inDirReference
);

/*dsGetPathFromList

Gets the path from a data list.

char * dsGetPathFromList (
   tDirReference inDirReference,
   const tDataList *inDataList,
   const char *inDelimiter
);

Parameters
inDirReference

On input, a value of type tDirReference obtained by calling dsOpenDirService or dsOpenDirServiceProxy representing the Open Directory session associated with the data list from which a path is to be obtained, or zero.
inDataList

On input, a pointer to a value of type tDataList containing the path to get.
inDelimiter

On input, a pointer to a character string containing the character that delimits the components of the path in the data list pointed to by the inDataList parameter.
function result

A pointer to a character string that contains the path that was obtained from the data list.
Discussion

This utility function gets the path from a data list. The path is in UTF-8 format.
Availability
Available in Mac OS X v10.0 and later.
Declared In
DirServicesUtils.h*/

char * dsGetPathFromList (
   tDirReference inDirReference,
   const tDataList *inDataList,
   const char *inDelimiter
);

/*dsCopyDirStatusName


char* dsCopyDirStatusName (
    SInt32 inDirStatus ) ;  
Discussion 

Pass in the tDirStatus enum and receive the string of the enum name. Memory for the char* is the responsibility of the client and can be cleaned up using free().

Availability
Introduced in Mac OS X v10.4.*/
char* dsCopyDirStatusName (
    SInt32 inDirStatus ) ;  

/*dsBuildListFromStringsAlloc

Fills in a previously allocated data list using data from strings.

tDirStatus dsBuildListFromStringsAlloc (
   tDirReference inDirReferences,
   tDataListPtr inDataList,
   const char *in1stCString,
   ...
);
Parameters
inDirReference

On input, a value of type tDirReference obtained by calling dsOpenDirService or dsOpenDirServiceProxy representing the Open Directory session that is associated with the specified data list, or zero.
inDataList

On input, a value of type tDataListPtr that points to a data list allocated by calling dsDataListAllocate.
in1stCString

On input, a pointer to a character string that specifies the name of a data node to add to the data list. The in1stCString parameter may be followed by one or more additional parameters of type char *, each pointing to a C string containing data that is to be added to the data list.
function result

A value of type tDirStatus indicating success (eDSNoErr) or an error. For a list of possible result codes, see “Result Codes.”
Discussion

This utility function fills in a data list using the data in UTF-8 format contained by the specified null-terminated strings.

When you no longer need the data list, call dsDataListDeallocate to release the memory associated with it. If the data list is a heap-based data list, you also need to call free().
Availability
Available in Mac OS X v10.0 and later.
Declared In
DirServicesUtils.h*/
tDirStatus dsBuildListFromStringsAlloc (
   tDirReference inDirReferences,
   tDataListPtr inDataList,
   const char *in1stCString,
   ...
);

/*dsBuildListFromStringsAllocV

Fills in a previously allocated data list using data from a list of type va_list.

tDirStatus dsBuildListFromStringsAllocV (
   tDirReference inDirRef,
   tDataList *inDataList,
   const char *in1stCString,
   va_list args
);

Parameters
inDirRef

On input, a value of type tDirReference obtained by calling dsOpenDirService representing the Open Directory session that is associated with the specified data list, or zero.
inDataList

On input, a pointer to a value of type tDataList allocated by calling dsDataListAllocate.
in1stCString

On input, a pointer to a character string that specifies the name of a data node to add to the data list. The in1stCString parameter may be followed by one or more additional parameters of type char *, each pointing to a C string containing data that is to be added to the data list.
args

On input, a value of type va_list with additional C strings containing data that is to be added to the date list.
function result

A value of type tDirStatus indicating success (eDSNoErr) or an error. For a list of possible result codes, see “Result Codes.”
Discussion

This utility function fills in a data list using the data in UTF-8 format contained by the specified null-terminated string and additional strings in the va_list parameter.

When you no longer need the data list, call dsDataListDeallocate to release the memory associated with it.
Availability
Available in Mac OS X v10.0 and later.
Declared In
DirServicesUtils.h*/
tDirStatus dsBuildListFromStringsAllocV (
   tDirReference inDirRef,
   tDataList *inDataList,
   const char *in1stCString,
   va_list args
);

/*dsDataNodeAllocateBlock

Allocates an Open Directory data node.

tDataNodePtr dsDataNodeAllocateBlock (
   tDirReference inDirReference,
   UInt32 inDataNodeSize,
   UInt32 inDataNodeLength,
   tBuffer inDataNodeBuffer
);

Parameters
inDirReference

On input, a value of type tDirReference obtained by calling dsOpenDirService or dsOpenDirServiceProxy representing the Open Directory session for which the data node is to be allocated, or zero.
inDataNodeSize

On input, a value of type unsigned long that specifies the size of inDataNodeBuffer.
inDataNodeLength

On input, a value of type unsigned long that specifies the length of valid data in inDataNodeBuffer.
inDataNodeBuffer

On input, a value of type tBuffer containing the value the data node is to contain.
function result

A value of type tDataNodePtr that points to the allocated data node and that can be passed as a parameter to Open Directory functions that require such a value as a parameter. If this function cannot allocate the data node, it returns NULL.
Discussion

This utility function allocates an Open Directory data node and returns a pointer to it. Use the data node as a convenient way to pass data, such as record names and authentication types, to Open Directory functions.

To release the memory associated with a data node, call dsDataNodeDeAllocate.

To use a C string to allocate a data node, call dsDataNodeAllocateString.
Availability
Available in Mac OS X v10.0 and later.
Declared In
DirServicesUtils.h*/
tDataNodePtr dsDataNodeAllocateBlock (
   tDirReference inDirReference,
   UInt32 inDataNodeSize,
   UInt32 inDataNodeLength,
   tBuffer inDataNodeBuffer
);


#ifdef __cplusplus
}
#endif

#endif //DIRSERVICESUTILS_H

