#ifndef __IOKIT_IOWORKLOOP_H
#define __IOKIT_IOWORKLOOP_H

#include <libkern/c++/OSObject.h>
#include <IOKit/IOReturn.h>
#include <IOKit/IOLib.h>
#include <IOKit/IOLocks.h>

#include <IOKit/system.h>

#include <HYS/IOKit/IOWorkLoop.h>
#ifdef __cplusplus
class IOEventSource;
class IOTimerEventSource;
class IOCommandGate;

class IOWorkLoop : public OSObject,public IOWorkLoop_HYS
{
    OSDeclareDefaultStructors(IOWorkLoop)

protected:

/*free


protected

virtual void free(); 
Discussion 

Mandatory free of the object independent of the current retain count. If the work loop is running, this method will not return until the thread has successfully terminated. Each event source in the chain will be released and the working semaphore will be destroyed. 

If the client has some outstanding requests on an event they will never be informed of completion. If an external thread is blocked on any of the event sources they will be awakened with a KERN_INTERUPTED status.*/
virtual void free(); 

/*gateLock


protected

IORecursiveLock *gateLock; 
Discussion 

Mutual exclusion lock that is used by close and open Gate functions. This is a recursive lock, which allows multiple layers of code to share a single IOWorkLoop without deadlock. This is common in IOKit since threads of execution tend to follow the service plane in the IORegistry, and multiple objects along the call path may acquire the gate for the same (shared) workloop.*/
IORecursiveLock *gateLock; 

public:

/*getThread


Gets the workThread. 

public

virtual IOThread getThread() const; 
Return Value

Returns workThread.*/
virtual IOThread getThread() const; 

/*onThread


Is the current execution context on the work thread? 

public

virtual bool onThread() const; 
Return Value

Returns true if IOThreadSelf() == workThread.*/
virtual bool onThread() const; 

/*inGate


Is the current execution context holding the work-loop's gate? 

public

virtual bool inGate() const; 
Return Value

Returns true if IOThreadSelf() is gate holder.*/

virtual bool inGate() const; 

/*disableAllInterrupts


Calls disable() in all interrupt event sources. 

public

virtual void disableAllInterrupts() const; 
Discussion 

For all event sources (ES) for which IODynamicCast(IOInterruptEventSource, ES) is valid, in eventChain call disable() function. See IOEventSource::disable().*/
virtual void disableAllInterrupts() const; 

/*enableAllInterrupts


Calls enable() in all interrupt event sources. 

public

virtual void enableAllInterrupts() const; 
Discussion 

For all event sources (ES) for which IODynamicCast(IOInterruptEventSource, ES) is valid, in eventChain call enable() function. See IOEventSource::enable().*/

virtual void enableAllInterrupts() const; 

/*disableAllEventSources


Calls disable() in all event sources. 

public

virtual void disableAllEventSources() const; 
Discussion 

For all event sources in eventChain, call disable() function. See IOEventSource::disable().*/
virtual void disableAllEventSources() const; 

/*Action


public

typedef IOReturn ( *Action)(
    OSObject *target, 
    void *arg0,
    void *arg1, 
    void *arg2,
    void *arg3); 
Fields
target
Target of the function, can be used as a refcon. Note if a C++ function was specified, this parameter is implicitly the first parameter in the target member function's parameter list.
arg0
Argument to action from run operation.
arg1
Argument to action from run operation.
arg2
Argument to action from run operation.
arg3
Argument to action from run operation.
Discussion 

Type and arguments of callout C function that is used when a runCommand is executed by a client. Cast to this type when you want a C++ member function to be used. Note the arg1 - arg3 parameters are straight pass through from the runCommand to the action callout.*/

typedef IOReturn ( *Action)(
    OSObject *target, 
    void *arg0,
    void *arg1, 
    void *arg2,
    void *arg3); 

/*removeEventSource

public

virtual IOReturn removeEventSource(
    IOEventSource *toRemove); 
Parameters
toRemove
Pointer to IOEventSource subclass to remove.
Return Value

Returns kIOReturnSuccess if successful, kIOReturnBadArgument if toRemove couldn't be found. 
Discussion 

Remove an event source from the work loop. This function does not return until the work loop has acknowledged the removal of the event source. When an event has been removed the threadMain will always restart its loop and check all outstanding events. The event source will be released before return.*/
virtual IOReturn removeEventSource(
    IOEventSource *toRemove); 

/*addEventSource

public

virtual IOReturn addEventSource(
    IOEventSource *newEvent); 
Parameters
newEvent
Pointer to IOEventSource subclass to add.
Return Value

Always returns kIOReturnSuccess. 
Discussion 

Add an event source to be monitored by the work loop. This function does not return until the work loop has acknowledged the arrival of the new event source. When a new event has been added the threadMain will always restart its loop and check all outstanding events. The event source is retained by the work loop.*/
virtual IOReturn addEventSource(
    IOEventSource *newEvent); 

/*workLoop


Factory member function to construct and intialize a work loop. 

public

static IOWorkLoop *workLoop(); 
Return Value

Returns a workLoop instance if constructed successfully, 0 otherwise.*/
static IOWorkLoop *workLoop(); 

/*runAction


Single thread a call to an action with the work-loop. 

public

OSMetaClassDeclareReservedUsed(
    IOWorkLoop, 0); 
Parameters
action
Pointer to function to be executed in work-loop context.
arg0
Parameter for action parameter, defaults to 0.
arg1
Parameter for action parameter, defaults to 0.
arg2
Parameter for action parameter, defaults to 0.
arg3
Parameter for action parameter, defaults to 0.
Return Value

Returns the value of the Action callout. 
Discussion 

Client function that causes the given action to be called in a single threaded manner. Beware: the work-loop's gate is recursive and runAction can cause direct or indirect re-entrancy. When executing on a client's thread, runAction will sleep until the work-loop's gate opens for execution of client actions, the action is single threaded against all other work-loop event sources.*/
OSMetaClassDeclareReservedUsed(
    IOWorkLoop, 0); 

    virtual IOReturn runAction(Action action, OSObject *target,
			       void *arg0 = 0, void *arg1 = 0,
			       void *arg2 = 0, void *arg3 = 0);
};

#endif//__cplusplus

#endif//__IOKIT_IOWORKLOOP_H
