#ifndef OS_OSMETACLASS_H
#define OS_OSMETACLASS_H

#include <sys/types.h>

#include <libkern/OSReturn.h>
#include <kern/debug.h>

//functions

#ifdef __cplusplus
class OSMetaClass;
class OSObject;
class OSString;
class OSSymbol;
class OSDictionary;
class OSSerialize;
#endif //__cplusplus

#include <HYS/libkern/c++/OSMetaClass.h>

#ifdef __cplusplus
class OSMetaClassBase:public OSMetaClassBase_HYS
{

private:

/*operator =


Disables implicit copy constructor by making private. 

private

void operator =(
    OSMetaClassBase &src); 
Parameters
src
Reference to source object that isn't allowed to be copied.*/
void operator =(
    OSMetaClassBase &src); 

public://macro

/*OSTypeAlloc


Allocate an instance of the desired object.

public

#define OSTypeAlloc(type)  ((type *) ((type::metaClass)->alloc()))
Parameters
type
Name of the desired type to be created.
Return Value

'this' if object cas been successfully created. 
Discussion 

The OSTypeAlloc macro can be used to break the binary compatibility difficulties presented by new. The problem is that C++ compiles the knowledge of the size of the class into the cade calling new. If you use the alloc code however the class size is determined by the callee not the caller.*/
#define OSTypeAlloc(type)  ((type *) ((type::metaClass)->alloc()))

/*OSDynamicCast


Roughly analogous to (type *) inst, but check if valid first.
public

#define OSDynamicCast(type, inst) \ 
    ((type *) OSMetaClassBase::safeMetaCast((inst), (type::metaClass))) 
Parameters
type
Name of desired class name. Notice that it is assumed that you desire to cast to a pointer to an object of this type. Also type qualifiers, like const, are not recognized and will cause a (usually obscure) compile error.
inst
Pointer to object that you wish to attempt to type cast. May be 0.
Return Value

If object non-zero and it is of the desired type, otherwise 0. 
Discussion 

OSDynamicCast is an attempt to implement a rudimentary equivalent to RTTI's dynamic_cast operator. Note that the restricted subset of C++ upon which the I/O Kit is based does not support RTTI. OSDynamicCast is build on the OSMetaClass mechanism. Note it is safe to call this with a 0 parameter.*/
#define OSDynamicCast(type, inst) \
    ((type *) OSMetaClassBase::safeMetaCast((inst), (type::metaClass))) 



/*OSMemberFunctionCast


Converts a pointer to a member function to a C-style pointer to function (note that no warnings are generated).

public

#define OSMemberFunctionCast(cptrtype, self, func) \ 
    (cptrtype) OSMetaClassBase:: \ 
    _ptmf2ptf(self, (void (OSMetaClassBase::*)(void)) func) 
Parameters
cptrtype
The type of pointer function desired.
self
The 'this' pointer of the object whose function you wish to cache.
func
The pointer to the member function itself, something like &Base::func.
Return Value

Returns a pointer to function of the given type. This function will panic if an attempt is made to call it with a multiply inherited class.*/
#define OSMemberFunctionCast(cptrtype, self, func) \
    (cptrtype) OSMetaClassBase:: \
    _ptmf2ptf(self, (void (OSMetaClassBase::*)(void)) func)

/*OSDefineMetaClassAndAbstractStructorsWithInit


Defines an OSMetaClass subclass and the primary constructors and destructors for a subclass of OSObject that is an abstract class.

public

#define OSDefineMetaClassAndAbstractStructorsWithInit(className, superClassName, init) \ 
    // Class global data  \ 
    className ::MetaClass className ::gMetaClass; \ 
    const OSMetaClass * const className ::metaClass = \ 
    & className ::gMetaClass; \ 
    const OSMetaClass * const className ::superClass = \ 
    & superClassName ::gMetaClass; \ 
    // Class member functions * \ 
    className :: className(const OSMetaClass *meta) \ 
    : superClassName (meta) { } \ 
    className ::~ className() { } \ 
    const OSMetaClass * className ::getMetaClass() const \ 
    { return &gMetaClass; } \ 
    // The ::MetaClass constructor  \ 
    className ::MetaClass::MetaClass() \ 
    : OSMetaClass("className", className::superClass, sizeof(className)) \ 
    { init; } \ 
    OSObject * className ::MetaClass::alloc() const { return ; } 
Parameters
className
Name of class (NO QUOTES and NO MACROS).
superClassName
Name of super class (NO QUOTES and NO MACROS).
init
Name of a function to call after the OSMetaClass is constructed.
Discussion 

This is the primary definition macro for all abstract classes that are subclasses of OSObject. In general this 'function' is 'called' at the top of the file just before the first function is implemented for a particular class. Once the OSMetaClass has been constructed, at load time, call the init routine. Note: you cannot rely on the order of execution of the init routines.*/

#define OSDefineMetaClassAndAbstractStructorsWithInit(className, superClassName, init) \
    OSDefineMetaClassWithInit(className, superClassName, init)		\
    OSDefineAbstractStructors(className, superClassName)

public://functions

/*metaCast(const OSMetaClass *)


Checks to see if this object is or inherits from the given type. 

public

OSMetaClassBase *metaCast(
    const OSMetaClass *toMeta) const; 
Parameters
toMeta
Pointer to a constant OSMetaClass for the desired target type.
Return Value

Returns 'this' if the object is of the desired type, otherwise 0. 
Discussion 

This function is the guts of the OSMetaClass system. IODynamicCast, qv, is implemented using this function. */
OSMetaClassBase *metaCast(
    const OSMetaClass *toMeta) const; 

/*metaCast(const OSString *)


See OSMetaClassBase::metaCast(const OSMetaClass *). 

public

OSMetaClassBase *metaCast(
    const OSString *toMeta) const; 
Parameters
toMeta
OSString of the desired class' name.
Return Value

Returns 'this' if the object is of the desired type, otherwise 0. */
OSMetaClassBase *metaCast(
    const OSString *toMeta) const; 

/*metaCast(const OSSymbol *)


See OSMetaClassBase::metaCast(const OSMetaClass *). 

public

OSMetaClassBase *metaCast(
    const OSSymbol *toMeta) const; 
Parameters
toMeta
OSSymbol of the desired class' name.
Return Value

Returns 'this' if the object is of the desired type, otherwise 0.*/
OSMetaClassBase *metaCast(
    const OSSymbol *toMeta) const; 

/*isEqualTo


Returns whether 'this' == anObj. 

public

virtual bool isEqualTo(
    const OSMetaClassBase *anObj) const; 
Parameters
anObj
Object to compare 'this' to.
Return Value

Returns true if the objects are equivalent, false otherwise. 
Discussion 

OSMetaClassBase::isEqualTo implements this as a shallow pointer comparison. The OS container classes do a more meaningful comparison. Your mileage may vary.*/
virtual bool isEqualTo(
    const OSMetaClassBase *anObj) const; 
/*metaCast(const char *)


See OSMetaClassBase::metaCast(const OSMetaClass *). 

public

OSMetaClassBase  *metaCast(
    const char *toMeta) const; 
Parameters
toMeta
const char * C String of the desired class' name.
Return Value

Returns 'this' if the object is of the desired type, otherwise 0.*/
OSMetaClassBase *metaCast(
    const char *toMeta) const; 

};

class OSMetaClass : private OSMetaClassBase,public OSMetaClass_HYS
{

public://functions

/*getSuperClass


'Get'ter for the super class. 

public

const OSMetaClass *getSuperClass() const; 
Return Value

Returns a pointer to superclass, chain ends with 0 for OSObject.*/
const OSMetaClass *getSuperClass() const; 

/*getClassName


'Get'ter for class name. 

public

const char *getClassName() const; 
Return Value

Returns a C string of the class name.*/
const char *getClassName() const; 

/*alloc


Allocates an instance of the class that this OSMetaClass instance represents. 

public

virtual OSObject *alloc() const = 0; 
Return Value

Returns a pointer to a new object with a retain count of 1. 
Discussion 

This alloc function is analogous to the old ObjC class alloc method. Typically not used by clients because the static function allocClassWithName is more generally useful. In fact, that function is implemented in terms of this virtual function. All subclasses of OSMetaClass must implement this function, but that is what the OSDefineMetaClassAndStructor's families of macros does for the developer automatically.*/
virtual OSObject *alloc() const = 0; 

/*allocClassWithName


Looks up a metaclass in the runtime type information system and returns the results of an alloc call. 

public

static OSObject *allocClassWithName(
    const char *name); 
Parameters
name
Name of the desired class.
Return Value

Returns a pointer to a new object, 0 if not found or no memory.*/
static OSObject *allocClassWithName(
    const char *name); 

/*OSMetaClass(const char *, const OSMetaClass *, unsigned int)


Constructor for OSMetaClass objects. 
protected

OSMetaClass(
    const char *inClassName, 
    const OSMetaClass *inSuperClass, 
    unsigned int inClassSize); 
Parameters
inClassName
C string of the name of the class this metaclass represents.
inSuperClassName
C string of the name of the super class.
inClassSize
size of the class.
Discussion 

This constructor is protected and cannot be used to instantiate an OSMetaClass object, i.e. OSMetaClass is an abstract class. This function stores the currently constructing OSMetaClass instance away for later processing. See preModLoad and postModLoad.*/
OSMetaClass(
    const char *inClassName, 
    const OSMetaClass *inSuperClass, 
    unsigned int inClassSize); 

/*instanceConstructed


Counts the instances of the class behind this metaclass. 
public

void instanceConstructed() const; 
Discussion 

Every non-abstract class that inherits from OSObject has a default constructor that calls its own metaclass's instanceConstructed function. This constructor is defined by the OSDefineMetaClassAndStructors macro (qv) that all OSObject subclasses must use. Also, if the instance count goes from 0 to 1, i.e. the first instance, then increment the instance count of the super class.*/
void instanceConstructed() const; 

public://macro


/*OSDefineMetaClassAndStructorsWithInit


See OSDefineMetaClassAndStructors.

public

#define OSDefineMetaClassAndStructorsWithInit(className, superClassName, init) \ 
    // Class global data  \ 
    className ::MetaClass className ::gMetaClass; \ 
    const OSMetaClass * const className ::metaClass = \ 
    & className ::gMetaClass; \ 
    const OSMetaClass * const className ::superClass = \ 
    & superClassName ::gMetaClass; \ 
    // Class member functions  \ 
    className :: className(const OSMetaClass *meta) \ 
    : superClassName (meta) { } \ 
    className ::~ className() { } \ 
    const OSMetaClass * className ::getMetaClass() const \ 
    { return &gMetaClass; } \ 
    // The ::MetaClass constructor  \ 
    className ::MetaClass::MetaClass() \ 
    : OSMetaClass("className", className::superClass, sizeof(className)) \ 
    { init; } \ 
    OSObject * className ::MetaClass::alloc() const \ 
    { return new className; } \ 
    className :: className () : superClassName (&gMetaClass) \ 
    { gMetaClass.instanceConstructed(); } 
Parameters
className
Name of class. NO QUOTES and NO MACROS.
superClassName
Name of super class. NO QUOTES and NO MACROS.
init
Name of a function to call after the OSMetaClass is constructed.
Discussion 

Define an OSMetaClass subclass and the primary constructors and destructors for a subclass of OSObject that isn't an abstract class. In general this 'function' is 'called' at the top of the file just before the first function is implemented for a particular class. Once the OSMetaClass has been constructed, at load time, call the init routine. NB you can not rely on the order of execution of the init routines.*/
#define OSDefineMetaClassAndStructorsWithInit(className, superClassName, init) \
    /* Class global data*/  \
    className ::MetaClass className ::gMetaClass; \
    const OSMetaClass * const className ::metaClass = \
    & className ::gMetaClass; \
    const OSMetaClass * const className ::superClass = \
    & superClassName ::gMetaClass; \
    /* Class member functions*/  \
    className :: className(const OSMetaClass *meta) \
    : superClassName (meta) { } \
    className ::~ className() { } \
    const OSMetaClass * className ::getMetaClass() const \
    { return &gMetaClass; } \
    // The ::MetaClass constructor  \
    className ::MetaClass::MetaClass() \
    : OSMetaClass("className", className::superClass, sizeof(className)) \
    { init; } \
    OSObject * className ::MetaClass::alloc() const \
    { return new className; } \
    className :: className () : superClassName (&gMetaClass) \
    { gMetaClass.instanceConstructed(); } 

/*OSDeclareDefaultStructors


Declares runtime type information data and interfaces (one of the macros used in the class declaration of all subclasses of OSObject). 
public

#define OSDeclareDefaultStructors(className) \ 
    private: \ 
    static const OSMetaClass * const superClass; \ 
    public: \ 
    static const OSMetaClass * const metaClass; \ 
    static class MetaClass : public OSMetaClass { \ 
    public: \ 
    MetaClass(); \ 
    virtual OSObject *alloc() const; \ 
    } gMetaClass; \ 
    friend class className ::MetaClass; \ 
    virtual const OSMetaClass * getMetaClass() const; \ 
    protected: \ 
    className (const OSMetaClass *); \ 
    virtual ~ className (); \ 
    public: \ 
    className (); \ 
    protected: 
Parameters
className
Name of class (NO QUOTES).
Discussion 

By convention this should be 'called' immediately after the opening brace in a class declaration. It leaves the current privacy state as 'protected:'.*/
#define OSDeclareDefaultStructors(className) \
    private: \
    static const OSMetaClass * const superClass; \
    public: \
    static const OSMetaClass * const metaClass; \
    static class MetaClass : public OSMetaClass { \
    public: \
    MetaClass(); \
    virtual OSObject *alloc() const; \
    } gMetaClass; \
    friend class className ::MetaClass; \
    virtual const OSMetaClass * getMetaClass() const; \
    protected: \
    className (const OSMetaClass *); \
    virtual ~ className (); \
    public: \
    className (); \
    protected:


/*OSDefineMetaClassWithInit


DO NOT USE. Basic helper macro for the OSDefineMetaClass for the default and Abstract macros, qv.

public

#define OSDefineMetaClassWithInit(className, superClassName, init) \ 
    // Class global data *
    className ::MetaClass className ::gMetaClass; \ 
    const OSMetaClass * const className ::metaClass = \ 
    & className ::gMetaClass; \ 
    const OSMetaClass * const className ::superClass = \ 
    & superClassName ::gMetaClass; \ 
    //Class member functions *
    className :: className(const OSMetaClass *meta) \ 
    : superClassName (meta) { } \ 
    className ::~ className() { } \ 
    const OSMetaClass * className ::getMetaClass() const \ 
    { return &gMetaClass; } \ 
    // The ::MetaClass constructor /\ 
    className ::MetaClass::MetaClass() \ 
    : OSMetaClass(#className, className::superClass, sizeof(className)) \ 
    { init; } 
Parameters
className
Name of class (NO QUOTES and NO MACROS).
superClassName
Name of super class (NO QUOTES and NO MACROS).
init
Name of a function to call after the OSMetaClass is constructed.*/

#define OSDefineMetaClassWithInit(className, superClassName, init) \
    /* Class global data */ \
    className ::MetaClass className ::gMetaClass; \
    const OSMetaClass * const className ::metaClass = \
    & className ::gMetaClass; \
    const OSMetaClass * const className ::superClass = \
    & superClassName ::gMetaClass; \
    /* Class member functions */ \
    className :: className(const OSMetaClass *meta) \
    : superClassName (meta) { } \
    className ::~ className() { } \
    const OSMetaClass * className ::getMetaClass() const \
    { return &gMetaClass; } \
    /* The ::MetaClass constructor */ \
    className ::MetaClass::MetaClass() \
    : OSMetaClass(#className, className::superClass, sizeof(className)) \
    { init; }

/*OSDefineDefaultStructors


DO NOT USE. Basic helper macro for the OSDefineMetaClass for the default and Abstract macros, qv.

public

#define OSDefineDefaultStructors(className, superClassName) \ 
    OSObject * className ::MetaClass::alloc() const \ 
    { return new className; } \ 
    className :: className () : superClassName (&gMetaClass) \ 
    { gMetaClass.instanceConstructed(); } 
Parameters
className
Name of class. NO QUOTES and NO MACROS.
superClassName
Name of super class. NO QUOTES and NO MACROS.*/

#define OSDefineDefaultStructors(className, superClassName) \
    OSObject * className ::MetaClass::alloc() const \
    { return new className; } \
    className :: className () : superClassName (&gMetaClass) \
    { gMetaClass.instanceConstructed(); }

/*OSDefineMetaClassAndStructors


Defines an OSMetaClass subclass and the runtime system routines.
public

#define OSDefineMetaClassAndStructors(className, superClassName) \ 
    // Class global data * \ 
    className ::MetaClass className ::gMetaClass; \ 
    const OSMetaClass * const className ::metaClass = \ 
    & className ::gMetaClass; \ 
    const OSMetaClass * const className ::superClass = \ 
    & superClassName ::gMetaClass; \ 
    // Class member functions * \ 
    className :: className(const OSMetaClass *meta) \ 
    : superClassName (meta) { } \ 
    className ::~ className() { } \ 
    const OSMetaClass * className ::getMetaClass() const \ 
    { return &gMetaClass; } \ 
    // The ::MetaClass constructor \ 
    className ::MetaClass::MetaClass() \ 
    : OSMetaClass(#className, className::superClass, sizeof(className)) \ 
    { ; } \ 
    OSObject * className ::MetaClass::alloc() const \ 
    { return new className; } \ 
    className :: className () : superClassName (&gMetaClass) \ 
    { gMetaClass.instanceConstructed(); } 
Parameters
className
Name of class. NO QUOTES and NO MACROS.
superClassName
Name of super class. NO QUOTES and NO MACROS.
Discussion 

Define an OSMetaClass subclass and the primary constructors and destructors for a subclass of OSObject that isn't an abstract class. In general this 'function' is 'called' at the top of the file just before the first function is implemented for a particular class.*/

#define OSDefineMetaClassAndStructors(className, superClassName) \
    /* Class global data */ \
    className ::MetaClass className ::gMetaClass; \
    const OSMetaClass * const className ::metaClass = \
    & className ::gMetaClass; \
    const OSMetaClass * const className ::superClass = \
    & superClassName ::gMetaClass; \
    /* Class member functions */ \
    className :: className(const OSMetaClass *meta) \
    : superClassName (meta) { } \
    className ::~ className() { } \
    const OSMetaClass * className ::getMetaClass() const \
    { return &gMetaClass; } \
    /* The ::MetaClass constructor */ \
    className ::MetaClass::MetaClass() \
    : OSMetaClass(#className, className::superClass, sizeof(className)) \
    { ; } \
    OSObject * className ::MetaClass::alloc() const \
    { return new className; } \
    className :: className () : superClassName (&gMetaClass) \
    { gMetaClass.instanceConstructed(); }



/*OSDeclareAbstractStructors


Declares runtime type information data and interfaces (one of the macros used in the class declaration of all subclasses of OSObject). 
public

#define OSDeclareAbstractStructors(className) \ 
    private: \ 
    static const OSMetaClass * const superClass; \ 
    public: \ 
    static const OSMetaClass * const metaClass; \ 
    static class MetaClass : public OSMetaClass { \ 
    public: \
    MetaClass(); \ 
    virtual OSObject *alloc() const; \ 
    } gMetaClass; \ 
    friend class className ::MetaClass; \ 
    virtual const OSMetaClass * getMetaClass() const; \ 
    protected: \ 
    className (const OSMetaClass *); \ 
    virtual ~ className (); \ 
    private: \ 
    className (); / Make primary constructor private in abstract *\ 
    protected: 
Parameters
className
Name of class (NO QUOTES).
Discussion 

This macro is used when the class being declared has one or more '= 0' pure virtual methods and thus it is illegal to create an instance of this class. It leaves the current privacy state as 'protected:'.*/
#define OSDeclareAbstractStructors(className) \
    private: \
    static const OSMetaClass * const superClass; \
    public: \
    static const OSMetaClass * const metaClass; \
    static class MetaClass : public OSMetaClass { \
    public: \
    MetaClass(); \
    virtual OSObject *alloc() const; \
    } gMetaClass; \
    friend class className ::MetaClass; \
    virtual const OSMetaClass * getMetaClass() const; \
    protected: \
    className (const OSMetaClass *); \
    virtual ~ className (); \
    private: \
    className (); /* Make primary constructor private in abstract */ \
    protected:


/*OSDefineMetaClassAndAbstractStructors 


Defines an OSMetaClass subclass and the runtime system routines.

public

#define OSDefineMetaClassAndAbstractStructors(className, superClassName) \ 
    // Class global data 
    className ::MetaClass className ::gMetaClass; \ 
    const OSMetaClass * const className ::metaClass = \ 
    & className ::gMetaClass; \ 
    const OSMetaClass * const className ::superClass = \ 
    & superClassName ::gMetaClass; \ 
    // Class member functions *\ 
    className :: className(const OSMetaClass *meta) \ 
    : superClassName (meta) { } \ 
    className ::~ className() { } \ 
    const OSMetaClass * className ::getMetaClass() const \ 
    { return &gMetaClass; } \ 
    // The ::MetaClass constructor \ 
    className ::MetaClass::MetaClass() \ 
    : OSMetaClass("className", className::superClass, sizeof(className)) \ 
    { ; } \ 
    OSObject * className ::MetaClass::alloc() const { return ; } 
Parameters
className 
Name of class. NO QUOTES and NO MACROS.
superClassName 
Name of super class. NO QUOTES and NO MACROS.
Discussion 

Define an OSMetaClass subclass and the primary constructors and destructors for a subclass of OSObject that is an abstract class. In general this 'function' is 'called' at the top of the file just before the first function is implemented for a particular class.*/

#define OSDefineMetaClassAndAbstractStructors(className, superClassName)\
    /* Class global data */ \
    className ::MetaClass className ::gMetaClass;\
    const OSMetaClass * const className ::metaClass = \
    & className ::gMetaClass; \
    const OSMetaClass * const className ::superClass = \
    & superClassName ::gMetaClass; \
    /* Class member functions */ \
    className :: className(const OSMetaClass *meta) \
    : superClassName (meta) { } \
    className ::~ className() { } \
    const OSMetaClass * className ::getMetaClass() const \
    { return &gMetaClass; } \
    /* The ::MetaClass constructor */ \
    className ::MetaClass::MetaClass() \
    : OSMetaClass("className", className::superClass, sizeof(className))\
    { ; } \
    //OSObject * className ::MetaClass::alloc() const { return ; }

/*OSDefineMetaClass


Defines an OSMetaClass instance, used for backward compatiblility only.

public

#define OSDefineMetaClass(className, superClassName) \ 
    // Class global data  \ 
    className ::MetaClass className ::gMetaClass; \ 
    const OSMetaClass * const className ::metaClass = \ 
    & className ::gMetaClass; \ 
    const OSMetaClass * const className ::superClass = \ 
    & superClassName ::gMetaClass; \ 
    // Class member functions  \ 
    className :: className(const OSMetaClass *meta) \ 
    : superClassName (meta) { } \ 
    className ::~ className() { } \ 
    const OSMetaClass * className ::getMetaClass() const \ 
    { return &gMetaClass; } \ 
    // The ::MetaClass constructor  \ 
    className ::MetaClass::MetaClass() \ 
    : OSMetaClass("className", className::superClass, sizeof(className)) \ 
    { ; } 
Parameters
className
Name of class. NO QUOTES and NO MACROS.
superClassName
Name of super class. NO QUOTES and NO MACROS.*/
#define OSDefineMetaClass(className, superClassName) \
 OSDefineMetaClassWithInit(className,superClassName,)

/*OSDefineAbstractStructors


DO NOT USE. Basic helper macro for the OSDefineMetaClass for the default and Abstract macros, qv.

public

#define OSDefineAbstractStructors(className, superClassName) \ 
    OSObject * className ::MetaClass::alloc() const { return 0; } 
Parameters
className
Name of class (NO QUOTES and NO MACROS).
superClassName
Name of super class (NO QUOTES and NO MACROS).*/
#define OSDefineAbstractStructors(className, superClassName) \
    OSObject * className ::MetaClass::alloc() const { return 0; } 

};
#endif //__cplusplus

#endif//OS_OSMETACLASS_H
