#include <sys/fcntl.h>

/*OPEN(2)                     BSD System Calls Manual                    OPEN(2)

NAME
     open -- open or create a file for reading or writing

SYNOPSIS
     #include <fcntl.h>

     int
     open(const char *path, int oflag, ...);

DESCRIPTION
     The file name specified by path is opened for reading and/or writing, as specified by the argument
     oflag; the file descriptor is returned to the calling process.

     The oflag argument may indicate that the file is to be created if it does not exist (by specifying the
     O_CREAT flag).  In this case, open requires a third argument mode_t mode; the file is created with mode
     mode as described in chmod(2) and modified by the process' umask value (see umask(2)).

     The flags specified are formed by or'ing the following values:

           O_RDONLY        open for reading only
           O_WRONLY        open for writing only
           O_RDWR          open for reading and writing
           O_NONBLOCK      do not block on open or for data to become available
           O_APPEND        append on each write
           O_CREAT         create file if it does not exist
           O_TRUNC         truncate size to 0
           O_EXCL          error if O_CREAT and the file exists
           O_SHLOCK        atomically obtain a shared lock
           O_EXLOCK        atomically obtain an exclusive lock
           O_NOFOLLOW      do not follow symlinks
           O_SYMLINK       allow open of symlinks

     Opening a file with O_APPEND set causes each write on the file to be appended to the end.  If O_TRUNC
     is specified and the file exists, the file is truncated to zero length.  If O_EXCL is set with O_CREAT
     and the file already exists, open() returns an error.  This may be used to implement a simple exclu-sive-access exclusive-access
     sive-access locking mechanism.  If O_EXCL is set and the last component of the pathname is a symbolic
     link, open() will fail even if the symbolic link points to a non-existent name.

     If the O_NONBLOCK flag is specified, do not wait for the device or file to be ready or available.  If
     the open() call would result in the process being blocked for some reason (e.g., waiting for carrier on
     a dialup line), open() returns immediately.  This flag also has the effect of making all subsequent I/O
     on the open file non-blocking.

     When opening a file, a lock with flock(2) semantics can be obtained by setting O_SHLOCK for a shared
     lock, or O_EXLOCK for an exclusive lock.  If creating a file with O_CREAT, the request for the lock
     will never fail (provided that the underlying filesystem supports locking).

     If O_NOFOLLOW is used in the mask and the target file passed to open() is a symbolic link then the
     open() will fail.

     If O_SYMLINK is used in the mask and the target file passed to open() is a symbolic link then the
     open() will be for the symbolic link itself, not what it links to.

     If successful, open() returns a non-negative integer, termed a file descriptor.  It returns -1 on fail-ure. failure.
     ure.  The file pointer (used to mark the current position within the file) is set to the beginning of
     the file.

     When a new file is created, it is given the group of the directory which contains it.

     The new descriptor is set to remain open across execve system calls; see close(2) and fcntl(2).

     The system imposes a limit on the number of file descriptors that can be held open simultaneously by
     one process.  Getdtablesize(2) returns the current system limit.

RETURN VALUES
     If successful, open() returns a non-negative integer, termed a file descriptor.  It returns -1 on fail-ure, failure,
     ure, and sets errno to indicate the error.

ERRORS
     The named file is opened unless:

     [EACCES]           Search permission is denied for a component of the path prefix.

     [EACCES]           The required permissions (for reading and/or writing) are denied for the given
                        flags.

     [EACCES]           O_CREAT is specified, the file does not exist, and the directory in which it is to
                        be created does not permit writing.

     [EACCES]           O_TRUNC is specified and write permission is denied.

     [EAGAIN]           path specifies the slave side of a locked pseudo-terminal device.

     [EDQUOT]           O_CREAT is specified, the file does not exist, and the directory in which the entry
                        for the new file is being placed cannot be extended because the user's quota of disk
                        blocks on the file system containing the directory has been exhausted.

     [EDQUOT]           O_CREAT is specified, the file does not exist, and the user's quota of inodes on the
                        file system on which the file is being created has been exhausted.

     [EEXIST]           O_CREAT and O_EXCL are specified and the file exists.

     [EFAULT]           Path points outside the process's allocated address space.

     [EINTR]            The open() operation is interrupted by a signal.

     [EINVAL]           The value of oflag is not valid.

     [EIO]              An I/O error occurs while making the directory entry or allocating the inode for
                        O_CREAT.

     [EISDIR]           The named file is a directory, and the arguments specify that it is to be opened for
                        writing.

     [ELOOP]            Too many symbolic links are encountered in translating the pathname.  This is taken
                        to be indicative of a looping symbolic link.

     [EMFILE]           The process has already reached its limit for open file descriptors.

     [ENAMETOOLONG]     A component of a pathname exceeds {NAME_MAX} characters, or an entire path name
                        exceeded {PATH_MAX} characters.

     [ENFILE]           The system file table is full.

     [ELOOP]            O_NOFOLLOW was specified and the target is a symbolic link.

     [ENOENT]           O_CREAT is not set and the named file does not exist.

     [ENOENT]           A component of the path name that must exist does not exist.

     [ENOSPC]           O_CREAT is specified, the file does not exist, and the directory in which the entry
                        for the new file is being placed cannot be extended because there is no space left
                        on the file system containing the directory.

     [ENOSPC]           O_CREAT is specified, the file does not exist, and there are no free inodes on the
                        file system on which the file is being created.

     [ENOTDIR]          A component of the path prefix is not a directory.

     [ENXIO]            The named file is a character-special or block-special file and the device associ-ated associated
                        ated with this special file does not exist.

     [ENXIO]            O_NONBLOCK and O_WRONLY are set, the file is a FIFO, and no process has it open for
                        reading.

     [EOPNOTSUPP]       O_SHLOCK or O_EXLOCK is specified, but the underlying filesystem does not support
                        locking.

     [EOPNOTSUPP]       An attempt is made to open a socket (not currently implemented).

     [EOVERFLOW]        The named file is a regular file and its size does not fit in an object of type
                        off_t.

     [EROFS]            The named file resides on a read-only file system, and the file is to be modified.

     [ETXTBSY]          The file is a pure procedure (shared text) file that is being executed and the
                        open() call requests write access.

COMPATIBILITY
     open() on a terminal device (i.e., /dev/console) will now make that device a controlling terminal for
     the process.  Use the O_NOCTTY flag to open a terminal device without changing your controlling termi-nal. terminal.
     nal.

SEE ALSO
     chmod(2), close(2), dup(2), getdtablesize(2), lseek(2), read(2), umask(2), write(2)

HISTORY
     An open() function call appeared in Version 6 AT&T UNIX.

4th Berkeley Distribution      November 16, 1993     4th Berkeley Distribution*/
   int      open(const char *path, int oflag, ...){};
